:- module(html_format_handle, 
	[
	    html_format/4,
	    html_format_input_shape/2,
	    html_format_input_dimensions/2,
	    load_html_format/2
	], [assertions, basicmodes, pillow]).

:- use_module(library(read)).
:- use_module(library(streams)).
:- use_module(library(terms), [atom_concat/2]).

:- data html_format/5.

:- pred load_html_form(Name, Dir) # "Loads html_format/5 facts from
     file @var{Name}.pl in directory @var{Dir} into the Prolog
     database. Can be called at any time, during execution or startup,
     allowing to change representation of html form fields.".

load_html_format(Name, Dir):-
	retractall_fact(html_format(_,_,_,_,_)),
	atom_concat([Dir, Name, '.pl'], File),
	open_input(File, IO), !,
	read(Fact),
	process_file(Fact),
	close_input(IO).
load_html_format(_,_).

process_file(end_of_file):- !.
process_file(Fact):-
	assertz_fact(Fact),
	read(NewFact),
	process_file(NewFact).


:- pred html_format(+Format, +Field, +DisplayMode, -Output) # "Given a
	format identification @var{Format} corresponding to an HTML
	data representation, a form field @var{Field} to be formatted,
	and a display mode @var{DisplayMode} defining whether the
	output is specific of admin or common users, produces
	associated HTML code.".

html_format(Format, Field, DisplayMode, Output) :-
	html_format(Format, Field, DisplayMode, Output, _Dimension).


:- pred html_format_input_shape(+Format, -Shape) # "Given a format or
	a format name name @var{Format} produces its dimension
	@var{Shape} shape. This can be either 'unidimensional' or
	'bidimensional', referring to the nature of the input form
	field it contains. For example an input text is regarded as
	unidimensional and a text area is considered bidimensional. If
	this criteria is not applicable a 'plain' is returned.".

html_format_input_shape(Format, Dimension) :-
	html_format(Format, _Field, _DisplayMode, _Output, Dimension).
html_format_input_shape(FormatName, Dimension) :-
	html_format(Format, _Field, _DisplayMode, _Output, Dimension),
	functor(Format, FormatName,_).

:- pred html_format_input_dimensions(+Format, -Dimensions) # "Obtains
	the dimensions @var{Dimensions} associated to the given format
	@var{Format}.".

html_format_input_dimensions(Format, uni(Cols)) :-
	html_format_input_shape(Format, unidimensional),
	arg(2, Format, Cols).
html_format_input_dimensions(Format, bi(Rows, Cols)) :-
	html_format_input_shape(Format, bidimensional),
	arg(2, Format, Rows),
	arg(3, Format, Cols).
html_format_input_dimensions(Format, plain) :-
	html_format_input_shape(Format, plain).
